#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def debian_package_install(packages):
    """Jinja utility method for building debian-based package install command.

    apt-get is not capable of installing .deb files from a URL and the
    template logic to construct a series of steps to install regular packages
    from apt repos as well as .deb files that need to be downloaded, manually
    installed, and cleaned up is complicated. This method will construct the
    proper string required to install all packages in a way that's a bit
    easier to follow.

    :param packages: a list of strings that are either packages to install
    from an apt repo, or URLs to .deb files
    :type packages: list

    :returns: string suitable to provide to RUN command in a Dockerfile that
    will install the given packages
    :rtype: string
    """
    cmds = []

    # divide the list into two groups, one for regular packages and one for
    # URL packages
    reg_packages, url_packages = [], []
    for package in packages:
        if package.startswith('http'):
            url_packages.append(package)
        else:
            reg_packages.append(package)

    # handle the apt-get install
    if reg_packages:
        cmds.append('apt-get update')
        cmds.append('apt-get -y install --no-install-recommends {}'.format(
            ' '.join(reg_packages)
        ))
        cmds.append('apt-get clean')
        cmds.append('rm -rf /var/lib/apt/lists/*')

    # handle URL packages
    for url in url_packages:
        # the path portion should be the file name
        name = url[url.rfind('/') + 1:]
        cmds.extend([
            'curl --location {} -o {}'.format(url, name),
            'dpkg -i {}'.format(name),
            'rm -rf {}'.format(name),
        ])

    # return the list of commands
    return ' && '.join(cmds)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re


mutable_default_args = re.compile(r"^\s*def .+\((.+=\{\}|.+=\[\])")

# D703: Found use of _() without explicit import of _!

UNDERSCORE_IMPORT_FILES = []

string_translation = re.compile(r"[^_]*_\(\s*('|\")")
translated_log = re.compile(
    r"(.)*LOG\.(audit|error|info|warn|warning|critical|exception)"
    "\(\s*_\(\s*('|\")")
underscore_import_check = re.compile(r"(.)*import _(.)*")
# We need this for cases where they have created their own _ function.
custom_underscore_check = re.compile(r"(.)*_\s*=\s*(.)*")


def check_explicit_underscore_import(logical_line, filename):
    """Check for explicit import of the _ function

    We need to ensure that any files that are using the _() function
    to translate logs are explicitly importing the _ function.  We
    can't trust unit test to catch whether the import has been
    added so we need to check for it here.

    """

    # Build a list of the files that have _ imported.  No further
    # checking needed once it is found.
    if filename in UNDERSCORE_IMPORT_FILES:
        pass
    elif (underscore_import_check.match(logical_line) or
          custom_underscore_check.match(logical_line)):
        UNDERSCORE_IMPORT_FILES.append(filename)
    elif (translated_log.match(logical_line) or
          string_translation.match(logical_line)):
        yield(0, "D703: Found use of _() without explicit import of _!")


def no_log_warn(logical_line):
    """Disallow 'LOG.warn('

    Deprecated LOG.warn(), instead use LOG.warning
    https://bugs.launchpad.net/senlin/+bug/1508442
    K302
    """

    msg = "K302: LOG.warn is deprecated, please use LOG.warning!"
    if "LOG.warn(" in logical_line:
        yield (0, msg)


def no_mutable_default_args(logical_line):
    msg = "K301: Method's default argument shouldn't be mutable!"
    if mutable_default_args.match(logical_line):
        yield (0, msg)


def factory(register):
    register(check_explicit_underscore_import)
    register(no_mutable_default_args)
    register(no_log_warn)



#!/usr/bin/python2
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import glob
import logging
import os
import re
import sys

import rrdtool


LOG = logging.getLogger('root')

COLORS = (
    '#0000ff',
    '#00ff00',
    '#ff0000',
    '#00ffff',
    '#ffff00',
    '#ff00ff'
)


def colorgen():
    for color in COLORS:
        yield color


def get_uptime_seconds():
    with open('/proc/uptime') as f:
        return f.read().split(' ')[0]


class LoadGraph(object):

    DEF = 'DEF:%(vname)s=%(filename)s:%(ds)s:%(CF)s'
    LINE = 'LINE1:%(vname)s%(color)s:%(vname)s'
    AREA = 'AREA:%(vname)s%(color)s:%(vname)s'

    def __init__(self, conf, file, color):
        self.conf = conf
        self.color = colorgen()
        self.file = file
        self.info = RRDInfo(file)
        self.color = color
        self.filename = os.path.basename(self.file)

    def _gen_line(self, vname, ds, color):
        params = dict(vname=vname,
                      ds=ds,
                      filename=self.file,
                      color=color,
                      CF=self.conf.cf)
        ret = []
        ret.append(self.DEF % params)
        draw = self.LINE
        if self.conf.area:
            draw = self.AREA
        if self.conf.stack:
            draw += ':STACK'
        ret.append(draw % params)
        LOG.info('Get params for ds: %s', ret)
        return ret

    def gen_params(self):

        ret = []
        for DS in self.info.data_sources():
            filename, ext = os.path.splitext(self.filename)
            vname = '%s-%s' % (filename, DS)
            ret.extend(self._gen_line(vname, DS, next(self.color)))
        return ret

    def graph(self):
        params = self.common_params()
        params += self.get_period()
        params.extend(self.gen_params())
        return rrdtool.graphv(*params)


class RRDInfo(object):

    def __init__(self, file):
        self.info = rrdtool.info(file)

    @property
    def last_update(self):
        return self.info['last_update']

    def data_sources(self):
        dss = set()
        for key in self.info:
            ds = re.findall('ds\[(\w*)', key)
            if ds:
                dss.add(ds[0])
        return dss


def main():
    logging.basicConfig(level=logging.DEBUG)

    parser = argparse.ArgumentParser()
    parser.add_argument('--file', nargs='+')
    parser.add_argument('--type')
    parser.add_argument('--height', default='256')
    parser.add_argument('--width', default='1024')
    parser.add_argument('--output', default='a.png')

    parser.add_argument('--stack', action='store_true')
    parser.add_argument('--area', action='store_true')
    parser.add_argument('--cf', default='MAX')

    parser.add_argument('--start', default=None)
    parser.add_argument('--end', default=None)

    conf = parser.parse_args(sys.argv[1:])

    last_update = None

    rrd_definitions = []

    color = colorgen()

    rrd_files = []
    for f in conf.file:
        f = os.path.expanduser(f)
        rrd_files.extend(glob.glob(f))
    LOG.info('Get rrdfiles: %s', rrd_files)

    for file in rrd_files:
        rrd_definitions.extend(LoadGraph(conf, file, color=color).gen_params())
        if not last_update:
            last_update = RRDInfo(file).last_update
        else:
            last_update = min(RRDInfo(file).last_update, last_update)

    start = conf.start
    end = conf.end
    if not start:
        start = 'end-' + get_uptime_seconds()
    if not end:
        end = str(last_update + 60)

    params = [
        conf.output,
        '--zoom', '4',
        '--width', conf.width,
        '--height', conf.height,
        '--start', start,
        '--end', end
        ]
    params.extend(rrd_definitions)
    LOG.debug('params: %s', params)

    rrdtool.graph(*params)


if __name__ == "__main__":
    main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from kolla.template import methods
from kolla.tests import base


class MethodsTest(base.TestCase):

    def test_debian_package_install(self):
        packages = ['https://packages.debian.org/package1.deb', 'package2.deb']
        result = methods.debian_package_install(packages)
        expectCmd = 'apt-get -y install --no-install-recommends package2.deb'
        self.assertEqual(expectCmd, result.split("&&")[1].strip())



#!/usr/bin/python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import subprocess  # nosec
import traceback


def extract_gospel_node(term):
    return term.split("@")[1].translate(None, "\'\"{},")


def main():
    try:
        # TODO(pbourke): see if can get gospel node without requiring shell
        raw_status = subprocess.check_output(
            "/usr/sbin/rabbitmqctl eval 'rabbit_clusterer:status().'",
            shell=True, stderr=subprocess.STDOUT  # nosec: this command appears
                                                  # to require a shell to work
        )
        if "Rabbit is running in cluster configuration" not in raw_status:
            raise AttributeError
        gospel_line = [
            line for line in raw_status.split('\n') if 'gospel' in line
        ][0]
        gospel_node = extract_gospel_node(gospel_line)
        if not gospel_node:
            raise AttributeError
    except AttributeError:
        result = {
            'failed': True,
            'error': raw_status,
            'changed': True
        }
    except Exception:
        result = {
            'failed': True,
            'error': traceback.format_exc(),
            'changed': True
        }
    else:
        result = {
            'failed': False,
            'hostname': gospel_node,
            'changed': False
        }

    print(json.dumps(result))


if __name__ == '__main__':
    main()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import multiprocessing
import os
import sys

from mock import patch
from oslo_log import fixture as log_fixture
from oslo_log import log as logging
from oslotest import base
import testtools


sys.path.append(
    os.path.abspath(os.path.join(os.path.dirname(__file__), '../tools')))
from kolla.image import build  # noqa

LOG = logging.getLogger(__name__)


class BuildTest(object):
    excluded_images = []

    def setUp(self):
        super(BuildTest, self).setUp()
        self.useFixture(log_fixture.SetLogLevel([__name__],
                                                logging.logging.INFO))

        self.threads = multiprocessing.cpu_count()
        if self.threads < 4:
            self.threads = 4

        self.build_args = [__name__, '--threads', str(self.threads)]

    @testtools.skipUnless(os.environ.get('DOCKER_BUILD_TEST'),
                          'Skip the docker build test')
    def runTest(self):
        with patch.object(sys, 'argv', self.build_args):
            LOG.info("Running with args %s", self.build_args)
            (bad_results, good_results, unmatched_results,
             skipped_results) = build.run_build()

        failures = 0
        for image, result in bad_results.items():
            if result is not 'error':
                continue
            failures = failures + 1
            LOG.critical(">>> Expected image '%s' to succeed!", image)

        for image in unmatched_results:
            LOG.warning(">>> Image '%s' was not matched", image)

        self.assertEqual(0, failures, "%d failure(s) occurred" % failures)


class BuildTestCentosBinary(BuildTest, base.BaseTestCase):

    def setUp(self):
        super(BuildTestCentosBinary, self).setUp()
        self.build_args.extend(["--base", "centos",
                                "--type", "binary"])


class BuildTestCentosSource(BuildTest, base.BaseTestCase):

    def setUp(self):
        super(BuildTestCentosSource, self).setUp()
        self.build_args.extend(["--base", "centos",
                                "--type", "source"])


class BuildTestUbuntuBinary(BuildTest, base.BaseTestCase):

    def setUp(self):
        super(BuildTestUbuntuBinary, self).setUp()
        self.build_args.extend(["--base", "ubuntu",
                                "--type", "binary"])


class BuildTestUbuntuSource(BuildTest, base.BaseTestCase):

    def setUp(self):
        super(BuildTestUbuntuSource, self).setUp()
        self.build_args.extend(["--base", "ubuntu",
                                "--type", "source"])


class BuildTestDebianBinary(BuildTest, base.BaseTestCase):

    def setUp(self):
        super(BuildTestDebianBinary, self).setUp()
        self.build_args.extend(["--base", "debian",
                                "--type", "binary"])


class BuildTestDebianSource(BuildTest, base.BaseTestCase):

    def setUp(self):
        super(BuildTestDebianSource, self).setUp()
        self.build_args.extend(["--base", "debian",
                                "--type", "source"])


class BuildTestOracleLinuxBinary(BuildTest, base.BaseTestCase):

    def setUp(self):
        super(BuildTestOracleLinuxBinary, self).setUp()
        self.build_args.extend(["--base", "oraclelinux",
                                "--type", "binary"])


class BuildTestOracleLinuxSource(BuildTest, base.BaseTestCase):

    def setUp(self):
        super(BuildTestOracleLinuxSource, self).setUp()
        self.build_args.extend(["--base", "oraclelinux",
                                "--type", "source"])


class DeployTestCentosBinary(BuildTestCentosBinary):

    def setUp(self):
        super(DeployTestCentosBinary, self).setUp()
        self.build_args.extend(["--profile", "gate"])


class DeployTestCentosSource(BuildTestCentosSource):

    def setUp(self):
        super(DeployTestCentosSource, self).setUp()
        self.build_args.extend(["--profile", "gate"])


class DeployTestOracleLinuxBinary(BuildTestOracleLinuxBinary):

    def setUp(self):
        super(DeployTestOracleLinuxBinary, self).setUp()
        self.build_args.extend(["--profile", "gate"])


class DeployTestOracleLinuxSource(BuildTestOracleLinuxSource):

    def setUp(self):
        super(DeployTestOracleLinuxSource, self).setUp()
        self.build_args.extend(["--profile", "gate"])


class DeployTestUbuntuBinary(BuildTestUbuntuBinary):

    def setUp(self):
        super(DeployTestUbuntuBinary, self).setUp()
        self.build_args.extend(["--profile", "gate"])


class DeployTestUbuntuSource(BuildTestUbuntuSource):

    def setUp(self):
        super(DeployTestUbuntuSource, self).setUp()
        self.build_args.extend(["--profile", "gate"])



#!/usr/bin/python

# Copyright 2015 Sam Yaple
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This module has been relicensed from the source below:
# https://github.com/SamYaple/yaodu/blob/master/ansible/library/ceph_osd_list

import json
import pyudev
import re
import subprocess  # nosec

from ansible.module_utils.basic import AnsibleModule

DOCUMENTATION = '''
---
module: find_disks
short_description: Return list of devices containing a specfied name or label
description:
     - This will return a list of all devices with either GPT partition name
       or filesystem label of the name specified.
options:
  match_mode:
    description:
      - Label match mode, either strict or prefix
    default: 'strict'
    required: False
    choices: [ "strict", "prefix" ]
    type: str
  name:
    description:
      - Partition name or filesystem label
    required: True
    type: str
    aliases: [ 'partition_name' ]
  use_udev:
    description:
      - When True, use Linux udev to read disk info such as partition labels,
        uuid, etc.  Some older host operating systems have issues using udev to
        get the info this module needs. Set to False to fall back to more low
        level commands such as blkid to retrieve this information. Most users
        should not need to change this.
    default: True
    required: False
    type: bool
author: Sam Yaple
'''

EXAMPLES = '''
- hosts: ceph-osd
  tasks:
    - name: Return all valid formated devices with the name KOLLA_CEPH_OSD
      find_disks:
          name: 'KOLLA_CEPH_OSD'
      register: osds

- hosts: swift-object-server
  tasks:
    - name: Return all valid devices with the name KOLLA_SWIFT
      find_disks:
          name: 'KOLLA_SWIFT'
      register: swift_disks

- hosts: swift-object-server
  tasks:
    - name: Return all valid devices with wildcard name 'swift_d*'
      find_disks:
          name: 'swift_d' match_mode: 'prefix'
      register: swift_disks
'''


PREFERRED_DEVICE_LINK_ORDER = [
    '/dev/disk/by-uuid',
    '/dev/disk/by-partuuid',
    '/dev/disk/by-parttypeuuid',
    '/dev/disk/by-label',
    '/dev/disk/by-partlabel'
]


def get_id_part_entry_name(dev, use_udev):
    if use_udev:
        dev_name = dev.get('ID_PART_ENTRY_NAME', '')
    else:
        part = re.sub(r'.*[^\d]', '', dev.device_node)
        parent = dev.find_parent('block').device_node
        # NOTE(Mech422): Need to use -i as -p truncates the partition name
        out = subprocess.Popen(['/usr/sbin/sgdisk', '-i', part,  # nosec
                                parent],
                               stdout=subprocess.PIPE).communicate()
        match = re.search(r'Partition name: \'(\w+)\'', out[0])
        if match:
            dev_name = match.group(1)
        else:
            dev_name = ''
    return dev_name


def get_id_fs_uuid(dev, use_udev):
    if use_udev:
        id_fs_uuid = dev.get('ID_FS_UUID', '')
    else:
        out = subprocess.Popen(['/usr/sbin/blkid', '-o', 'export',  # nosec
                                dev.device_node],
                               stdout=subprocess.PIPE).communicate()
        match = re.search(r'\nUUID=([\w-]+)', out[0])
        if match:
            id_fs_uuid = match.group(1)
        else:
            id_fs_uuid = ''
    return id_fs_uuid


def is_dev_matched_by_name(dev, name, mode, use_udev):
    if dev.get('DEVTYPE', '') == 'partition':
        dev_name = get_id_part_entry_name(dev, use_udev)
    else:
        dev_name = dev.get('ID_FS_LABEL', '')

    if mode == 'strict':
        return dev_name == name
    elif mode == 'prefix':
        return dev_name.startswith(name)
    else:
        return False


def find_disk(ct, name, match_mode, use_udev):
    for dev in ct.list_devices(subsystem='block'):
        if is_dev_matched_by_name(dev, name, match_mode, use_udev):
            yield dev


def get_device_link(dev):
    for preferred_link in PREFERRED_DEVICE_LINK_ORDER:
        for link in dev.device_links:
            if link.startswith(preferred_link):
                return link
    return dev.device_node


def extract_disk_info(ct, dev, name, use_udev):
    if not dev:
        return
    kwargs = dict()
    kwargs['fs_uuid'] = get_id_fs_uuid(dev, use_udev)
    kwargs['fs_label'] = dev.get('ID_FS_LABEL', '')
    if dev.get('DEVTYPE', '') == 'partition':
        kwargs['partition_label'] = name
        kwargs['device'] = dev.find_parent('block').device_node
        kwargs['partition'] = dev.device_node
        kwargs['partition_num'] = \
            re.sub(r'.*[^\d]', '', dev.device_node)
        if is_dev_matched_by_name(dev, name, 'strict', use_udev):
            kwargs['external_journal'] = False
            # NOTE(jeffrey4l): this is only used for bootstrap osd stage and
            # there is no journal partion at all. So it is OK to use
            # device_node directly.
            kwargs['journal'] = dev.device_node[:-1] + '2'
            kwargs['journal_device'] = kwargs['device']
            kwargs['journal_num'] = 2
        else:
            kwargs['external_journal'] = True
            journal_name = get_id_part_entry_name(dev, use_udev) + '_J'
            for journal in find_disk(ct, journal_name, 'strict', use_udev):
                kwargs['journal'] = get_device_link(journal)
                kwargs['journal_device'] = \
                    journal.find_parent('block').device_node
                kwargs['journal_num'] = \
                    re.sub(r'.*[^\d]', '', journal.device_node)
                break
            if 'journal' not in kwargs:
                # NOTE(SamYaple): Journal not found, not returning info
                return
    else:
        kwargs['device'] = dev.device_node
    yield kwargs


def extract_disk_info_bs(ct, dev, name, use_udev):
    if not dev:
        return
    kwargs = dict(bs_blk_label='', bs_blk_device='', bs_db_label='',
                  bs_db_device='', bs_wal_label='', bs_wal_device='',
                  bs_wal_partition_num='', bs_db_partition_num='',
                  bs_blk_partition_num='', partition='', partition_label='',
                  partition_num='', device='', partition_usage='')
    kwargs['fs_uuid'] = get_id_fs_uuid(dev, use_udev)
    kwargs['fs_label'] = dev.get('ID_FS_LABEL', '')

    if dev.get('DEVTYPE', '') == 'partition':
        actual_name = get_id_part_entry_name(dev, use_udev)

        if (('BOOTSTRAP_BS' in name or 'DATA_BS' in name)
                and name in actual_name):
            if actual_name.endswith("_B"):
                kwargs['partition_usage'] = 'block'
                kwargs['bs_blk_partition_num'] = \
                    re.sub(r'.*[^\d]', '', dev.device_node)
                kwargs['bs_blk_device'] = dev.find_parent('block').device_node
                kwargs['bs_blk_label'] = actual_name
                return kwargs
            if actual_name.endswith("_D"):
                kwargs['partition_usage'] = 'block.db'
                kwargs['bs_db_partition_num'] = \
                    re.sub(r'.*[^\d]', '', dev.device_node)
                kwargs['bs_db_device'] = dev.find_parent('block').device_node
                kwargs['bs_db_label'] = actual_name
                return kwargs
            if actual_name.endswith("_W"):
                kwargs['partition_usage'] = 'block.wal'
                kwargs['bs_wal_partition_num'] = \
                    re.sub(r'.*[^\d]', '', dev.device_node)
                kwargs['bs_wal_device'] = dev.find_parent('block').device_node
                kwargs['bs_wal_label'] = actual_name
                return kwargs
            if '_BS' in actual_name:
                kwargs['partition_usage'] = 'osd'
                kwargs['partition'] = dev.find_parent('block').device_node
                kwargs['partition_label'] = actual_name
                kwargs['partition_num'] = \
                    re.sub(r'.*[^\d]', '', dev.device_node)
                kwargs['device'] = dev.find_parent('block').device_node
                return kwargs
    return 0


def nb_of_osd(disks):
    osd_info = dict()
    osd_info['block_label'] = list()
    nb_of_osds = 0
    for item in disks:
        if item['partition_usage'] == 'osd':
            osd_info['block_label'].append(item['partition_label'])
            nb_of_osds += 1
    osd_info['nb_of_osd'] = nb_of_osds
    return osd_info


def combine_info(disks):
    info = list()
    osds = nb_of_osd(disks)
    osd_id = 0
    while osd_id < osds['nb_of_osd']:
        final = dict()
        idx = 0
        idx_osd = idx_blk = idx_wal = idx_db = -1
        for item in disks:
            if (item['partition_usage'] == 'osd' and
                    item['partition_label'] == osds['block_label'][osd_id]):
                idx_osd = idx
            elif (item['partition_usage'] == 'block' and
                    item['bs_blk_label'] ==
                    osds['block_label'][osd_id] + "_B"):
                idx_blk = idx
            elif (item['partition_usage'] == 'block.wal' and
                    item['bs_wal_label'] ==
                    osds['block_label'][osd_id] + "_W"):
                idx_wal = idx
            elif (item['partition_usage'] == 'block.db' and
                    item['bs_db_label'] ==
                    osds['block_label'][osd_id] + "_D"):
                idx_db = idx
            idx = idx + 1

        # write the information of block.db and block.wal to block item
        # if block.db and block.wal are found
        if idx_blk != -1:
            disks[idx_osd]['bs_blk_device'] = disks[idx_blk]['bs_blk_device']
            disks[idx_osd]['bs_blk_label'] = disks[idx_blk]['bs_blk_label']
            disks[idx_osd]['bs_blk_partition_num'] = \
                disks[idx_blk]['bs_blk_partition_num']
            disks[idx_blk]['partition_usage'] = ''
        if idx_wal != -1:
            disks[idx_osd]['bs_wal_device'] = disks[idx_wal]['bs_wal_device']
            disks[idx_osd]['bs_wal_partition_num'] = \
                disks[idx_wal]['bs_wal_partition_num']
            disks[idx_osd]['bs_wal_label'] = disks[idx_wal]['bs_wal_label']
            disks[idx_wal]['partition_usage'] = ''
        if idx_db != -1:
            disks[idx_osd]['bs_db_device'] = disks[idx_db]['bs_db_device']
            disks[idx_osd]['bs_db_partition_num'] = \
                disks[idx_db]['bs_db_partition_num']
            disks[idx_osd]['bs_db_label'] = disks[idx_db]['bs_db_label']
            disks[idx_db]['partition_usage'] = ''

        final['fs_uuid'] = disks[idx_osd]['fs_uuid']
        final['fs_label'] = disks[idx_osd]['fs_label']
        final['bs_blk_device'] = disks[idx_osd]['bs_blk_device']
        final['bs_blk_label'] = disks[idx_osd]['bs_blk_label']
        final['bs_blk_partition_num'] = disks[idx_osd]['bs_blk_partition_num']
        final['bs_db_device'] = disks[idx_osd]['bs_db_device']
        final['bs_db_partition_num'] = disks[idx_osd]['bs_db_partition_num']
        final['bs_db_label'] = disks[idx_osd]['bs_db_label']
        final['bs_wal_device'] = disks[idx_osd]['bs_wal_device']
        final['bs_wal_partition_num'] = disks[idx_osd]['bs_wal_partition_num']
        final['bs_wal_label'] = disks[idx_osd]['bs_wal_label']
        final['device'] = disks[idx_osd]['device']
        final['partition'] = disks[idx_osd]['partition']
        final['partition_label'] = disks[idx_osd]['partition_label']
        final['partition_num'] = disks[idx_osd]['partition_num']
        final['external_journal'] = False
        final['journal'] = ''
        final['journal_device'] = ''
        final['journal_num'] = 0

        info.append(final)
        disks[idx_osd]['partition_usage'] = ''
        osd_id += 1

    return info


def main():
    argument_spec = dict(
        match_mode=dict(required=False, choices=['strict', 'prefix'],
                        default='strict'),
        name=dict(aliases=['partition_name'], required=True, type='str'),
        use_udev=dict(required=False, default=True, type='bool')
    )
    module = AnsibleModule(argument_spec)
    match_mode = module.params.get('match_mode')
    name = module.params.get('name')
    use_udev = module.params.get('use_udev')

    try:
        ret = list()
        ct = pyudev.Context()
        for dev in find_disk(ct, name, match_mode, use_udev):
            if '_BS' in name:
                info = extract_disk_info_bs(ct, dev, name, use_udev)
                if info:
                    ret.append(info)
            else:
                for info in extract_disk_info(ct, dev, name, use_udev):
                    if info:
                        ret.append(info)

        if '_BS' in name and len(ret) > 0:
            ret = combine_info(ret)

        module.exit_json(disks=json.dumps(ret))
    except Exception as e:
        module.exit_json(failed=True, msg=repr(e))


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys


# NOTE(SamYaple): Update the search path to prefer PROJECT_ROOT as the source
#                 of packages to import if we are using local tools instead of
#                 pip installed kolla tools
PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '../..'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)

from kolla.image import build  # noqa


def main():
    statuses = build.run_build()
    if statuses:
        (bad_results, good_results, unmatched_results,
         skipped_results) = statuses
        if bad_results:
            return 1
    return 0


if __name__ == '__main__':
    sys.exit(main())



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import fixtures
import itertools
import mock
import os
import requests
import sys

from kolla.cmd import build as build_cmd
from kolla import exception
from kolla.image import build
from kolla.tests import base


FAKE_IMAGE = build.Image(
    'image-base', 'image-base:latest',
    '/fake/path', parent_name=None,
    parent=None, status=build.STATUS_MATCHED)
FAKE_IMAGE_CHILD = build.Image(
    'image-child', 'image-child:latest',
    '/fake/path2', parent_name='image-base',
    parent=FAKE_IMAGE, status=build.STATUS_MATCHED)
FAKE_IMAGE_CHILD_UNMATCHED = build.Image(
    'image-child-unmatched', 'image-child-unmatched:latest',
    '/fake/path3', parent_name='image-base',
    parent=FAKE_IMAGE, status=build.STATUS_UNMATCHED)
FAKE_IMAGE_CHILD_ERROR = build.Image(
    'image-child-error', 'image-child-error:latest',
    '/fake/path4', parent_name='image-base',
    parent=FAKE_IMAGE, status=build.STATUS_ERROR)
FAKE_IMAGE_CHILD_BUILT = build.Image(
    'image-child-built', 'image-child-built:latest',
    '/fake/path5', parent_name='image-base',
    parent=FAKE_IMAGE, status=build.STATUS_BUILT)


class TasksTest(base.TestCase):

    def setUp(self):
        super(TasksTest, self).setUp()
        self.image = FAKE_IMAGE.copy()
        # NOTE(jeffrey4l): use a real, temporary dir
        self.image.path = self.useFixture(fixtures.TempDir()).path
        self.imageChild = FAKE_IMAGE_CHILD.copy()
        # NOTE(mandre) we want the local copy of FAKE_IMAGE as the parent
        self.imageChild.parent = self.image
        self.imageChild.path = self.useFixture(fixtures.TempDir()).path

    @mock.patch('docker.version', '2.7.0')
    @mock.patch.dict(os.environ, clear=True)
    @mock.patch('docker.APIClient')
    def test_push_image_before_v3_0_0(self, mock_client):
        self.dc = mock_client
        pusher = build.PushTask(self.conf, self.image)
        pusher.run()
        mock_client().push.assert_called_once_with(
            self.image.canonical_name, decode=True,
            stream=True, insecure_registry=True)

    @mock.patch('docker.version', '3.0.0')
    @mock.patch.dict(os.environ, clear=True)
    @mock.patch('docker.APIClient')
    def test_push_image(self, mock_client):
        self.dc = mock_client
        pusher = build.PushTask(self.conf, self.image)
        pusher.run()
        mock_client().push.assert_called_once_with(
            self.image.canonical_name, decode=True, stream=True)

    @mock.patch.dict(os.environ, clear=True)
    @mock.patch('docker.APIClient')
    def test_build_image(self, mock_client):
        self.dc = mock_client
        push_queue = mock.Mock()
        builder = build.BuildTask(self.conf, self.image, push_queue)
        builder.run()

        mock_client().build.assert_called_once_with(
            path=self.image.path, tag=self.image.canonical_name, decode=True,
            network_mode=None, nocache=False, rm=True, pull=True, forcerm=True,
            buildargs=None)

        self.assertTrue(builder.success)

    @mock.patch.dict(os.environ, clear=True)
    @mock.patch('docker.APIClient')
    def test_build_image_with_network_mode(self, mock_client):
        self.dc = mock_client
        push_queue = mock.Mock()
        self.conf.set_override('network_mode', 'host')

        builder = build.BuildTask(self.conf, self.image, push_queue)
        builder.run()

        mock_client().build.assert_called_once_with(
            path=self.image.path, tag=self.image.canonical_name, decode=True,
            network_mode='host', nocache=False, rm=True, pull=True,
            forcerm=True, buildargs=None)

        self.assertTrue(builder.success)

    @mock.patch.dict(os.environ, clear=True)
    @mock.patch('docker.APIClient')
    def test_build_image_with_build_arg(self, mock_client):
        self.dc = mock_client
        build_args = {
            'HTTP_PROXY': 'http://localhost:8080',
            'NO_PROXY': '127.0.0.1'
        }
        self.conf.set_override('build_args', build_args)
        push_queue = mock.Mock()
        builder = build.BuildTask(self.conf, self.image, push_queue)
        builder.run()

        mock_client().build.assert_called_once_with(
            path=self.image.path, tag=self.image.canonical_name, decode=True,
            network_mode=None, nocache=False, rm=True, pull=True, forcerm=True,
            buildargs=build_args)

        self.assertTrue(builder.success)

    @mock.patch.dict(os.environ, {'http_proxy': 'http://FROM_ENV:8080'},
                     clear=True)
    @mock.patch('docker.APIClient')
    def test_build_arg_from_env(self, mock_client):
        push_queue = mock.Mock()
        self.dc = mock_client
        build_args = {
            'http_proxy': 'http://FROM_ENV:8080',
        }
        builder = build.BuildTask(self.conf, self.image, push_queue)
        builder.run()

        mock_client().build.assert_called_once_with(
            path=self.image.path, tag=self.image.canonical_name, decode=True,
            network_mode=None, nocache=False, rm=True, pull=True, forcerm=True,
            buildargs=build_args)

        self.assertTrue(builder.success)

    @mock.patch.dict(os.environ, {'http_proxy': 'http://FROM_ENV:8080'},
                     clear=True)
    @mock.patch('docker.APIClient')
    def test_build_arg_precedence(self, mock_client):
        self.dc = mock_client
        build_args = {
            'http_proxy': 'http://localhost:8080',
        }
        self.conf.set_override('build_args', build_args)

        push_queue = mock.Mock()
        builder = build.BuildTask(self.conf, self.image, push_queue)
        builder.run()

        mock_client().build.assert_called_once_with(
            path=self.image.path, tag=self.image.canonical_name, decode=True,
            network_mode=None, nocache=False, rm=True, pull=True, forcerm=True,
            buildargs=build_args)

        self.assertTrue(builder.success)

    @mock.patch('docker.APIClient')
    @mock.patch('requests.get')
    def test_requests_get_timeout(self, mock_get, mock_client):
        self.dc = mock_client
        self.image.source = {
            'source': 'http://fake/source',
            'type': 'url',
            'name': 'fake-image-base'
        }
        push_queue = mock.Mock()
        builder = build.BuildTask(self.conf, self.image, push_queue)
        mock_get.side_effect = requests.exceptions.Timeout
        get_result = builder.process_source(self.image, self.image.source)

        self.assertIsNone(get_result)
        self.assertEqual(self.image.status, build.STATUS_ERROR)
        mock_get.assert_called_once_with(self.image.source['source'],
                                         timeout=120)

        self.assertFalse(builder.success)

    @mock.patch('os.utime')
    @mock.patch('shutil.copyfile')
    @mock.patch('shutil.rmtree')
    @mock.patch('docker.APIClient')
    @mock.patch('requests.get')
    def test_process_source(self, mock_get, mock_client,
                            mock_rmtree, mock_copyfile, mock_utime):
        for source in [{'source': 'http://fake/source1', 'type': 'url',
                       'name': 'fake-image-base1',
                        'reference': 'http://fake/reference1'},
                       {'source': 'http://fake/source2', 'type': 'git',
                       'name': 'fake-image-base2',
                        'reference': 'http://fake/reference2'},
                       {'source': 'http://fake/source3', 'type': 'local',
                       'name': 'fake-image-base3',
                        'reference': 'http://fake/reference3'},
                       {'source': 'http://fake/source4', 'type': None,
                       'name': 'fake-image-base4',
                        'reference': 'http://fake/reference4'}]:
            self.image.source = source
            push_queue = mock.Mock()
            builder = build.BuildTask(self.conf, self.image, push_queue)
            get_result = builder.process_source(self.image, self.image.source)
            self.assertEqual(self.image.status, build.STATUS_ERROR)
            self.assertFalse(builder.success)
            if source['type'] != 'local':
                self.assertIsNone(get_result)
            else:
                self.assertIsNotNone(get_result)

    @mock.patch('os.path.exists')
    @mock.patch('os.utime')
    @mock.patch('shutil.rmtree')
    def test_process_git_source_existing_dir(self, mock_rmtree, mock_utime,
                                             mock_path_exists):
        source = {'source': 'http://fake/source1', 'type': 'git',
                  'name': 'fake-image1',
                  'reference': 'fake/reference1'}

        self.image.source = source
        self.image.path = "fake_image_path"
        mock_path_exists.return_value = True
        push_queue = mock.Mock()
        builder = build.BuildTask(self.conf, self.image, push_queue)
        get_result = builder.process_source(self.image, self.image.source)

        mock_rmtree.assert_called_with(
            "fake_image_path/fake-image1-archive-fake-reference1")
        self.assertEqual(self.image.status, build.STATUS_ERROR)
        self.assertFalse(builder.success)
        self.assertIsNone(get_result)

    @mock.patch('docker.APIClient')
    def test_followups_docker_image(self, mock_client):
        self.imageChild.source = {
            'source': 'http://fake/source',
            'type': 'url',
            'name': 'fake-image-base'
        }
        self.imageChild.children.append(FAKE_IMAGE_CHILD_UNMATCHED)
        push_queue = mock.Mock()
        builder = build.BuildTask(self.conf, self.imageChild, push_queue)
        builder.success = True
        self.conf.push = FAKE_IMAGE_CHILD_UNMATCHED
        get_result = builder.followups
        self.assertEqual(1, len(get_result))


class KollaWorkerTest(base.TestCase):

    config_file = 'default.conf'

    def setUp(self):
        super(KollaWorkerTest, self).setUp()
        image = FAKE_IMAGE.copy()
        image.status = None
        image_child = FAKE_IMAGE_CHILD.copy()
        image_child.status = None
        image_unmatched = FAKE_IMAGE_CHILD_UNMATCHED.copy()
        image_error = FAKE_IMAGE_CHILD_ERROR.copy()
        image_built = FAKE_IMAGE_CHILD_BUILT.copy()
        # NOTE(mandre) we want the local copy of FAKE_IMAGE as the parent
        for i in [image_child, image_unmatched, image_error, image_built]:
            i.parent = image

        self.images = [image, image_child, image_unmatched,
                       image_error, image_built]
        patcher = mock.patch('docker.APIClient')
        self.addCleanup(patcher.stop)
        self.mock_client = patcher.start()

    def test_supported_base_type(self):
        rh_base = ['centos', 'oraclelinux', 'rhel']
        rh_type = ['source', 'binary', 'rdo', 'rhos']
        deb_base = ['ubuntu', 'debian']
        deb_type = ['source', 'binary']

        for base_distro, install_type in itertools.chain(
                itertools.product(rh_base, rh_type),
                itertools.product(deb_base, deb_type)):
            self.conf.set_override('base', base_distro)
            self.conf.set_override('install_type', install_type)
            # should no exception raised
            build.KollaWorker(self.conf)

    def test_unsupported_base_type(self):
        for base_distro, install_type in itertools.product(
                ['ubuntu', 'debian'], ['rdo', 'rhos']):
            self.conf.set_override('base', base_distro)
            self.conf.set_override('install_type', install_type)
            self.assertRaises(exception.KollaMismatchBaseTypeException,
                              build.KollaWorker, self.conf)

    def test_build_image_list_adds_plugins(self):

        self.conf.set_override('install_type', 'source')

        kolla = build.KollaWorker(self.conf)
        kolla.setup_working_dir()
        kolla.find_dockerfiles()
        kolla.create_dockerfiles()
        kolla.build_image_list()
        expected_plugin = {
            'name': 'neutron-server-plugin-networking-arista',
            'reference': 'master',
            'source': 'https://git.openstack.org/openstack/networking-arista',
            'type': 'git'
        }

        found = False
        for image in kolla.images:
            if image.name == 'neutron-server':
                for plugin in image.plugins:
                    if plugin == expected_plugin:
                        found = True
                        break
                break
        if not found:
            self.fail('Can not find the expected neutron arista plugin')

    def test_build_image_list_plugin_parsing(self):
        """Ensure regex used to parse plugins adds them to the correct image"""
        self.conf.set_override('install_type', 'source')

        kolla = build.KollaWorker(self.conf)
        kolla.setup_working_dir()
        kolla.find_dockerfiles()
        kolla.create_dockerfiles()
        kolla.build_image_list()
        for image in kolla.images:
            if image.name == 'base':
                self.assertEqual(len(image.plugins), 0,
                                 'base image should not have any plugins '
                                 'registered')
                break
        else:
            self.fail('Expected to find the base image in this test')

    def test_set_time(self):
        self.conf.set_override('install_type', 'source')
        kolla = build.KollaWorker(self.conf)
        kolla.setup_working_dir()
        kolla.set_time()

    def _get_matched_images(self, images):
        return [image for image in images
                if image.status == build.STATUS_MATCHED]

    def test_skip_parents(self):
        self.conf.set_override('regex', ['image-child'])
        self.conf.set_override('skip_parents', True)
        kolla = build.KollaWorker(self.conf)
        kolla.images = self.images
        kolla.filter_images()

        self.assertEqual(build.STATUS_SKIPPED, kolla.images[1].parent.status)

    def test_without_profile(self):
        kolla = build.KollaWorker(self.conf)
        kolla.images = self.images
        kolla.filter_images()

        self.assertEqual(5, len(self._get_matched_images(kolla.images)))

    def test_build_rpm_setup(self):
        """checking the length of list of docker commands"""
        self.conf.set_override('rpm_setup_config', ["a.rpm", "b.repo"])
        kolla = build.KollaWorker(self.conf)
        self.assertEqual(2, len(kolla.rpm_setup))

    def test_pre_defined_exist_profile(self):
        # default profile include the fake image: image-base
        self.conf.set_override('profile', ['default'])
        kolla = build.KollaWorker(self.conf)
        kolla.images = self.images
        kolla.filter_images()

        self.assertEqual(1, len(self._get_matched_images(kolla.images)))

    def test_pre_defined_exist_profile_not_include(self):
        # infra profile do not include the fake image: image-base
        self.conf.set_override('profile', ['infra'])
        kolla = build.KollaWorker(self.conf)
        kolla.images = self.images
        kolla.filter_images()

        self.assertEqual(0, len(self._get_matched_images(kolla.images)))

    def test_pre_defined_not_exist_profile(self):
        # NOTE(jeffrey4l): not exist profile will raise ValueError
        self.conf.set_override('profile', ['not_exist'])
        kolla = build.KollaWorker(self.conf)
        kolla.images = self.images
        self.assertRaises(ValueError,
                          kolla.filter_images)

    @mock.patch('json.dump')
    def test_list_dependencies(self, dump_mock):
        self.conf.set_override('profile', ['all'])
        kolla = build.KollaWorker(self.conf)
        kolla.images = self.images
        kolla.filter_images()
        kolla.list_dependencies()
        dump_mock.assert_called_once_with(mock.ANY, sys.stdout, indent=2)

    def test_summary(self):
        kolla = build.KollaWorker(self.conf)
        kolla.images = self.images
        kolla.image_statuses_good['good'] = None
        kolla.image_statuses_bad['bad'] = None
        kolla.image_statuses_unmatched['unmatched'] = None
        results = kolla.summary()
        self.assertIsNone(results['failed'][0]['status'])

    @mock.patch('shutil.copytree')
    def test_work_dir(self, copytree_mock):
        self.conf.set_override('work_dir', 'tmp/foo')
        kolla = build.KollaWorker(self.conf)
        kolla.setup_working_dir()
        self.assertEqual('tmp/foo/docker', kolla.working_dir)


class MainTest(base.TestCase):

    @mock.patch.object(build, 'run_build')
    def test_images_built(self, mock_run_build):
        image_statuses = ({}, {'img': 'built'}, {}, {})
        mock_run_build.return_value = image_statuses
        result = build_cmd.main()
        self.assertEqual(0, result)

    @mock.patch.object(build, 'run_build')
    def test_images_unmatched(self, mock_run_build):
        image_statuses = ({}, {}, {'img': 'unmatched'}, {})
        mock_run_build.return_value = image_statuses
        result = build_cmd.main()
        self.assertEqual(0, result)

    @mock.patch.object(build, 'run_build')
    def test_no_images_built(self, mock_run_build):
        mock_run_build.return_value = None
        result = build_cmd.main()
        self.assertEqual(0, result)

    @mock.patch.object(build, 'run_build')
    def test_bad_images(self, mock_run_build):
        image_statuses = ({'img': 'error'}, {}, {}, {})
        mock_run_build.return_value = image_statuses
        result = build_cmd.main()
        self.assertEqual(1, result)

    @mock.patch('sys.argv')
    @mock.patch('docker.APIClient')
    def test_run_build(self, mock_client, mock_sys):
        result = build.run_build()
        self.assertTrue(result)

    @mock.patch.object(build, 'run_build')
    def test_skipped_images(self, mock_run_build):
        image_statuses = ({}, {}, {}, {'img': 'skipped'})
        mock_run_build.return_value = image_statuses
        result = build_cmd.main()
        self.assertEqual(0, result)



#!/usr/bin/python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import fnmatch
import json
import logging
import os
import re
import sys

import jinja2


PROJECT_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))

NEWLINE_EOF_INCLUDE_PATTERNS = ['*.j2', '*.yml', '*.py', '*.sh']
NEWLINE_EOF_EXCLUDE_PATTERNS = ['.tox', '.testrepository', '.git']

# Render json file by using jinja2 template is OK
JSON_J2_INCLUDE_PATTERNS = ['*.json.j2', '*.json']
JSON_J2_EXCLUDE_PATTERNS = ['.tox', '.testrepository', '.git']

logging.basicConfig()
LOG = logging.getLogger(__name__)


def check_newline_eof():
    includes = r'|'.join([fnmatch.translate(x)
                          for x in NEWLINE_EOF_INCLUDE_PATTERNS])
    excludes = r'|'.join([fnmatch.translate(x)
                          for x in NEWLINE_EOF_EXCLUDE_PATTERNS])
    return_code = 0

    def has_newline_eof(path):
        with open(path, 'r') as f:
            data = f.read()
            if data and data[-1] != '\n':
                LOG.error('%s file error: no newline at end of file', path)
                return False
        return True

    for root, dirs, files in os.walk(PROJECT_ROOT):
        dirs[:] = [d for d in dirs if not re.match(excludes, d)]
        for f in files:
            if not re.match(excludes, f) and re.match(includes, f):
                if not has_newline_eof(os.path.join(root, f)):
                    return_code = 1
    return return_code


def check_json_j2():
    includes = r'|'.join([fnmatch.translate(x)
                          for x in JSON_J2_INCLUDE_PATTERNS])
    excludes = r'|'.join([fnmatch.translate(x)
                          for x in JSON_J2_EXCLUDE_PATTERNS])
    return_code = 0

    def bool_filter(value):
        return True

    # Mock ansible hostvars variable, which is a nested dict
    def hostvars():
        return collections.defaultdict(hostvars)

    def validate_json_j2(root, filename):
        env = jinja2.Environment(  # nosec: not used to render HTML
            loader=jinja2.FileSystemLoader(root))
        env.filters['bool'] = bool_filter
        template = env.get_template(filename)
        # Mock ansible variables.
        context = {
            'hostvars': hostvars(),
            'cluster_interface': 'cluster_interface',
            'storage_interface': 'storage_interface',
            'inventory_hostname': 'hostname'
        }
        data = template.render(**context)
        json.loads(data)
    for root, dirs, files in os.walk(PROJECT_ROOT):
        dirs[:] = [d for d in dirs if not re.match(excludes, d)]
        for filename in files:
            if not re.match(excludes, filename) and \
                    re.match(includes, filename):
                fullpath = os.path.join(root, filename)
                try:
                    validate_json_j2(root, filename)
                except (ValueError, jinja2.exceptions.TemplateError):
                    return_code = 1
                    LOG.exception('%s file error', fullpath)
    return return_code


def main():
    checks = (
        check_newline_eof,
        check_json_j2
    )
    return sum([check() for check in checks])

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from jinja2 import contextfilter
from jinja2 import Undefined


@contextfilter
def customizable(context, val_list, call_type):
    # NOTE(mgoddard): Don't try to customise undefined values. There are cases
    # where this might happen, for example using a generic template overrides
    # file for building multiple image install types and/or distros, where
    # variables are not defined in every case.
    if isinstance(val_list, Undefined):
        return val_list
    name = context['image_name'].replace("-", "_") + "_" + call_type + "_"
    if name + "override" in context:
        return context[name + "override"]
    if name + "append" in context:
        val_list.extend(context[name + "append"])
    if name + "remove" in context:
        for removal in context[name + "remove"]:
            if removal in val_list:
                val_list.remove(removal)
    return val_list



#!/usr/bin/python

# Copyright 2015 Sam Yaple
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This is a stripped down version of an ansible module I wrote in Yaodu to
# achieve the same goals we have for Kolla. I have relicensed it for Kolla
# https://github.com/SamYaple/yaodu/blob/master/ansible/library/bslurp

# Basically this module will fetch the admin and mon keyrings as well as the
# monmap file. It then hashes the content, compresses them, and finally it
# converts them to base64 to be safely transported around with ansible

import base64
import hashlib
import json
import os
import sys
import zlib


def json_exit(msg=None, failed=False, changed=False):
    if type(msg) is not dict:
        msg = {'msg': str(msg)}
    msg.update({'failed': failed, 'changed': changed})
    print(json.dumps(msg))
    sys.exit()


def read_file(filename):
    filename_path = os.path.join('/etc/ceph', filename)

    if not os.path.exists(filename_path):
        json_exit("file not found: {}".format(filename_path), failed=True)
    if not os.access(filename_path, os.R_OK):
        json_exit("file not readable: {}".format(filename_path), failed=True)

    with open(filename_path, 'rb') as f:
        raw_data = f.read()

    return {'content': base64.b64encode(zlib.compress(raw_data)),
            'sha1': hashlib.sha1(raw_data).hexdigest(),
            'filename': filename}


def main():
    admin_keyring = 'ceph.client.admin.keyring'
    mon_keyring = 'ceph.client.mon.keyring'
    rgw_keyring = 'ceph.client.radosgw.keyring'
    monmap = 'ceph.monmap'

    files = [admin_keyring, mon_keyring, rgw_keyring, monmap]
    json_exit({filename: read_file(filename) for filename in files})


if __name__ == '__main__':
    main()
